@@ -19,6 +19,7 @@ import re
 import smtplib
 from subprocess import run
 import sys
+import time
 from tqdm.auto import tqdm
 import wordninja as wn
 
@@ -176,6 +177,11 @@ def hsplit(text, sep, group=True, attach=True):
     """Flexible string splitting that retains the delimiter rather, unlike
     the built-in str.split() method.
 
+    NOTE: I recently observed behavior suggesting separators with special
+    characters (e.g. "\n") may not work as expected for some settings. It
+    should work when group=True and attach=True though since I rewrote that
+    with new logic without the re module.
+
     Parameters
     -----------
     text: str
@@ -230,7 +236,7 @@ def hsplit(text, sep, group=True, attach=True):
     if group:
         # Subcase 0.1
         if attach:
-            return [word for word in re.findall(regex, text)][:-1]
+            return _grouped_split(text, sep)
 
         # Subcase 0.2
         else:
@@ -249,6 +255,26 @@ def hsplit(text, sep, group=True, attach=True):
     return [word for word in chain(*zip(words, [sep]*len(words))) if word][:-1]
 
 
+def _grouped_split(text, sep):
+    """Hsplit helper for case where group=True and attach=True (see hsplit
+    docs). Old re.find() method didn't work right when sep had special
+    characters (e.g. "\n").
+    """
+    res = []
+    toks = text.split(sep)
+    max_idx = len(toks) - 1
+    for i, tok in enumerate(toks):
+        if tok:
+            if i < max_idx: tok += sep
+            res.append(tok)
+        elif i < max_idx:
+            if res:
+                res[-1] += sep
+            else:
+                res.append(sep)
+    return res
+
+
 def rmvars(*args):
     """Wrapper to quickly free up memory by deleting global variables. Htools
     3.0 does not provide a way to do this for local variables.
@@ -905,6 +931,80 @@ def smap(*x):
     return amap('shape', *x)
 
 
+def sleepy_range(*args, wait=1, wait_before=True):
+    """Convenience function: we often want to create a loop that mimics doing
+    some time intensive thing on each iteration. This is just like the built-in
+    range function (not technically a function!) but with a sleep period baked
+    in, making it particularly useful for list comprehensions where this would
+    be tricky otherwise. Note: unlike range, calling this is destructive.
+    See examples.
+
+    Parameters
+    ----------
+    args: int
+        Passed on to range().
+    wait: int or float
+        Number of seconds to wait on each iteration. Remember this is a keyword
+        only argument for compatibility with the range interface.
+    wait_before: bool
+        Determines whether to sleep before or after yielding the number.
+        Defaults to before to mimic "doing work" before producing some result.
+
+    Examples
+    --------
+    # Takes 6 seconds to create this list.
+    >>> [i for i in sleepy_range(3, wait=2)]
+    [0, 1, 2]
+
+    >>> srange = sleepy_range(0, 6, 2, wait_before=False)
+    >>> for i in srange:
+    >>>     print(i)
+    0
+    2
+    4
+    >>> for i in srange:
+    >>>     print(i)
+
+    # Notice this cannot be used again without manually calling sleepy_range.
+    """
+    for i in range(*args):
+        if wait_before: time.sleep(wait)
+        yield i
+        if not wait_before: time.sleep(wait)
+
+
+def venumerate(iterable, start=0, freq=1, print_before=True,
+               message_format='{}'):
+    """Verbose enumerate: simple convenience function that's a drop-in
+    replacement for enumerate. It prints updates as we iterate over some
+    object. TQDM progress bar may not be available in some cases (e.g. we
+    don't know the length of the interval, or possible some cases using
+    concurrency?), and this function gives us some way to keep an eye on
+    progress. Mainly intended as a convenience for list comprehensions, since
+    in a standard for loop we could easily add this logic.
+
+    Parameters
+    ----------
+    iterable: Iterable
+        The object to iterate over.
+    start: int
+        Passed on to enumerate - the first index to use when counting.
+    freq: int
+        Frequency with which to print updates (i.e. updates are printed when
+        i is divisible by freq).
+    print_before: bool
+        Specifies whether to print the message before yielding the i'th value
+        or after.
+    message_format: str
+        Used to format the message that will be displayed when i is divisible
+        by freq. Defaults to just printing i.
+    """
+    for i, x in enumerate(iterable, start=start):
+        if i % freq == 0 and print_before: print(message_format.format(i))
+        yield i, x
+        if i % freq == 0 and not print_before: print(message_format.format(i))
+
+
 def method_of(meth):
     """Retrieve the class a method belongs to. This will NOT work on
     attributes. Also, this won't help if your goal is to retrieve an instance:
