@@ -21,8 +21,8 @@ import urllib
 import warnings
 from weakref import WeakSet
 
-from htools import hdir, load, save, identity, hasstatic, tolist, select, \
-    func_name
+from htools.core import hdir, load, save, identity, hasstatic, tolist,\
+    select, func_name
 from htools.config import STD_LIB_GIST
 
 
@@ -825,7 +825,6 @@ class AbstractAttrs(type):
         # abstract parent should not define the required attributes: it merely
         # enforces the requirement that its children do. We want the children
         # to inherit class_attrs and inst_attrs without overwriting them when
-        # their own __new__ is called, so that AbstractAttrs.__call__ can
         # use them for validation. Only change this if you're very confident
         # you understand the repercussions.
         if class_attrs or inst_attrs:
@@ -889,7 +888,7 @@ class Counted:
 
     >>> b = Bar(3)
     >>> b2 = Bar(3)
-    >>> b.instance_num, b2.instnce_num
+    >>> b.instance_num, b2.instance_num
     0, 1
     >>> Bar._instance_count
     2
@@ -2123,6 +2122,75 @@ def temporary_globals(func, **kwargs):
                 del func.__globals__[k]
 
 
+def global_functions(include_imported=False, include_ipy_like=False):
+    """Get all available functions in the current module.
+
+    Parameters
+    ----------
+    include_imported: bool
+        If True, include imported functions (this can be a LOT of functions if
+        we've used star imports, as recommended for certain libraries like
+        htools and fastai). If False, only return functions defined in the
+        current module.
+    include_ipy_like: bool
+        Specifies whether to include functions whose names are like "_243"
+        which is how IPython seems to store previously called functions
+        (or something like that).
+
+    Returns
+    -------
+    dict[str, FunctionType]: Dict mapping function name to function.
+    """
+    res = {}
+    for k, v in globals().items():
+        if not isinstance(v, types.FunctionType): continue
+        if not (include_imported or v.__module__ == '__main__'): continue
+        if not include_ipy_like and k.strip('_').isnumeric(): continue
+        res[k] = v
+    return res
+
+
+def decorate_functions(decorator, include_imported=False,
+                       include_ipy_like=True):
+    """Decorate all (or some large subset, depending on args) functions
+    available in the current module's global scope. Can be useful for
+    debugging (see examples).
+
+    Parameters
+    ----------
+    decorator: FunctionType
+        The function that will be used to decorate all available functions.
+        This must accept only a function as an argument, so make sure to pass
+        in the appropriate object (for instance, if you want to use the
+        htools.meta.timeboxed decorator, you must call it first with the
+        desired arguments (e.g. time).
+    include_imported: bool
+        If True, include imported functions (this can be a LOT of functions if
+        we've used star imports, as recommended for certain libraries like
+        htools and fastai). If False, only return functions defined in the
+        current module.
+    include_ipy_like: bool
+        Specifies whether to include functions whose names are like "_243"
+        which is how IPython seems to store previously called functions
+        (or something like that).
+
+    Examples
+    --------
+    def foo(a):
+        # Do something
+
+    def bar(x, y):
+        # Do something else
+
+    if __name__ == '__main__':
+        decorate_functions(debug)
+        foo(3)
+        bar(4, 5)
+    """
+    for k, v in global_functions(include_imported, include_ipy_like).items():
+        globals()[k] = decorator(v)
+
+
 def fallback(meth=None, *, keep=(), drop=(), save=False):
     """Make instance/class attributes available as default arguments for a
     method. Kwargs can be passed in to override one or more of them. You can
@@ -2463,36 +2531,41 @@ def function_interface(present=(), required=(), defaults=(), startswith=(),
     return decorator
 
 
-def lazy(func):
+def Lazy(func=None, *, lazy=True):
     """"Decorator that provides a function with a boolean parameter "lazy".
     When set to true, the function will not be executed yet, sort of like a
-    coroutine. (See examples.) This can be nice for testing purposes. Also
+    coroutine (see examples). This can be nice for testing purposes. Also
     opens the door to some interesting things (maybe sort of allows for
-    decorated objects? Not sure of all applications yet.).
+    decorated objects? Not sure of all applications yet). Would have preferred
+    a lowercase function name but I also want to keep the parameter name as
+    "lazy" while avoiding confusion.
 
     Examples
     --------
-    @lazy
+    @Lazy
     def foo(a, b=3):
         return a * b
 
-    >>> foo(2)
+    >>> foo(2, lazy=False)
     6
 
-    >>> res = foo(2, lazy=True)
+    >>> res = foo(2)
     >>> res()
     6
 
     In the second example, notice we didn't get any output until explicitly
-    calling the result.
+    calling the result. Also note that we can change the default mode by using
+    the decorator like (keyword argument, not positional):
+    @Lazy(lazy=False)
     """
+    if func is None: return partial(Lazy, lazy=lazy)
     if 'lazy' in params(func):
         raise RuntimeError(
             f'Decorated function {func} must not have parameter named "lazy".'
             'It will be inserted automatically.'
         )
     @wraps(func)
-    def wrapper(*args, lazy=False, **kwargs):
+    def wrapper(*args, lazy=lazy, **kwargs):
         if lazy:
             return lambda: func(*args, **kwargs)
         return func(*args, **kwargs)
